/* $Id: ccdcamtr.cpp,v 1.7 2007/07/23 09:36:30 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "ccdcamtr.h"

namespace DEP
{

// read camera data, camera has different image size!!!

bool CCDCameraTR::read(istream& is)
{
	bool rtn = CCDCamera::read(is);
	if (rtn != true) return rtn;

	// calc. image format
	_size_x = _nr * _sr;
	_size_y = _nc * _sc;

	return true;
}


// get camera type

string CCDCameraTR::type_str() const
{
	ostringstream oss;
	oss << "CCD-frame";
	return oss.str();
}


// get orientation type

string CCDCameraTR::orientation_str() const
{
	ostringstream oss;
	oss << "top-right";
	return oss.str();
}


// external-to-image-coords. transformation

void CCDCameraTR::external_to_image(const DPoint2& ext_c, DPoint2& img_c) const
{
	img_c.x = - (ext_c.y - _r0) * _sr;
	img_c.y = - (ext_c.x - _c0) * _sc;

	//cout << pix_c << "; " << img_c << endl;
	//cout << "CameraCCDTR::external_to_image" << endl;
	
}


// image-to-external-coords. transformation

void CCDCameraTR::image_to_external(const DPoint2& img_c, DPoint2& ext_c) const
{
	ext_c.x = _c0 - img_c.y / _sc;
	ext_c.y = _r0 - img_c.x / _sr;

	//cout << pix_c << "; " << img_c << endl;
	//cout << "CameraCCDTR::image_to_external" << endl;
}


// transformation of standard deviations from external to image coordinates

void CCDCameraTR::sdev_external_to_image(const DPoint2& ext_s,
DPoint2& img_s) const
{
	img_s.x = ext_s.x * _sr;
	img_s.y = ext_s.y * _sc;
}


// non-member functions ...

} // namespace DEP
