/* $Id: eoaustralis.h,v 1.9 2007/07/23 08:40:34 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_EOAUSTRALIS_INCLUDED
#define DEP_EOAUSTRALIS_INCLUDED

#include "eopokrot.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Exterior orientation using euler angles.
 *   
 *    Parameters            : 3 rotation angles (euler angles)
 *    Coordinate axes       : ?
 *    Sequence of rotations : Rz(r) * Rx(e) * Rz(a) ?
 *                            special Australis implementation
 *    Direction of rotation : counter-clockwise ?
 *
 *  @author Dirk Stallmann
 */
class EOAustralis : public EOAnglesPOKRot
{
public:
	/// constructor
	EOAustralis() : EOAnglesPOKRot() {}
	
	/**@name overridden base class members of AdjObject */
	//@{
		/// get type name
		string type_str() const;

		/// get key name
		string key_str() const;
	
		/// get parameter name
		string uname(size_t i) const;
	//@}

	/**@name resolved pure virtual methods of classo ExtOrientation */
	//@{
		/// calculate rotation parameters from rotation matrix
		void calc_rot_params(vector<double>& a, const DRotMatrix3& R) const
		{
			assert(a.size() == 3);
			R.get_angles_australis(a[0], a[1], a[2]);
		}

		/// calculate rotation matrix
		void calc_rot_matrix(DRotMatrix3& R, const vector<double>& a) const
		{
			R.set_angles_australis(a[0], a[1], a[2]);
		}
	//@}

private:
	/// calculate partial derivatives of the rotation matrix
	void dr(double az, double el, double ro,
	DRotMatrix3& dR_az, DRotMatrix3& dR_el, DRotMatrix3& dR_ro) const
	{
		derivs_australis(az, el, ro, dR_az, dR_el, dR_ro);
	}
};


// non-member functions ...

} // namespace DEP

#endif
