/* $Id: eopokrot.h,v 1.12 2007/07/23 08:41:55 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_EOPOKROT_INCLUDED
#define DEP_EOPOKROT_INCLUDED

#include "extori.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Exterior orientation using euler angles class.
 *   
 *    Parameters            : 3 rotation angles (euler angles)
 *    Coordinate axes       : rotated
 *    Sequence of rotations : Ry(phi) * Rx(omega) * Rz(kappa)
 *    Direction of rotation : counter-clockwise
 *
 *  @author Dirk Stallmann
 */
class EOAnglesPOKRot : public ExtOrientation
{
public:
	/// constructor
	EOAnglesPOKRot() : ExtOrientation(3) {}
	
	/**@name overridden base class members of AdjObject */
	//@{
		/// get type name
		virtual string type_str() const;

		/// get key name
		virtual string key_str() const;
	
		/// get parameter name
		virtual string uname(size_t i) const;

		// sum of absolute changes
		//void sum_changes(const Vector<double>& dx,
		//	double& sum, size_t& num) const {}
	//@}
	
	/**@name overridden class members of ExtOrientation */
	//@{
		/// read EO parameters
		bool read(istream& is);

		/// write EO parameters
		void write(ostream& os) const;

		/// print EO parameters
		void print(ostream& os) const;

		/// print changes of unknowns
		void print_changes(ostream& os, const Vector<double>& dx) const;

		/// print unknown parameters
		void print_unknowns(ostream& os) const;
	//@}
	
	/**@name resolved pure vitual methods of class ExtOrientation */
	//@{
		/// calculate rotation parameters from rotation matrix
		virtual void calc_rot_params(vector<double>& a, const DRotMatrix3& R) const
		{
			assert(a.size() == 3);
			R.get_angles_pok_rot(a[0], a[1], a[2]);
		}
		
		/// calculate rotation matrix
		virtual void calc_rot_matrix(DRotMatrix3& R, const vector<double>& a) const
		{
			assert(a.size() == 3);
			R.set_angles_pok_rot(a[0], a[1], a[2]);
		}
		
		/// calculate partial derivatives of rotation parameters
		void calc_derivs_rot(DerivSub& deriv,
			double h, double u, double v, double w,
			const DPoint3& d, const vector<double>& a) const;
	
		/// calculate partial derivatives of rotation parameters
		void calc_derivs_rot(DerivSub& deriv,
			double h, double u, double v, double w,
			const DPoint3& d, const DRotMatrix3& M, const vector<double>& a) const;
	//@}
	
private:
	/// calculate partial derivatives of the rotation matrix
	virtual void dr(double phi, double omega, double kappa,
	DRotMatrix3& dR_ph, DRotMatrix3& dR_om, DRotMatrix3& dR_ka) const
	{
		derivs_pok_rot_ccw(phi, omega, kappa, dR_ph, dR_om, dR_ka);
	}
};


// non-member functions ...

} // namespace DEP

#endif
