/**********************************************************************/
//
// FILE:    frameext.h
//          Extened frame image class.
// CREATED: 23 Feb 05 (ds)
// CHANGES: 
// REFS:    
//
// (c) 2005 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#ifndef DEP_FRAMEEXT_INCLUDED
#define DEP_FRAMEEXT_INCLUDED

#include "image.h"

using namespace GenLib2;

namespace DEP
{
class PosCorrection;
class AttCorrection;
class IMUMisalignment;

/** Image class.
 *  Frame image
 *  Ties everything together which belongs to an (simple) image:
 *  image points, image geometry and imaging camera. 
 *
 *  @author Dirk Stallmann
 */
class ExtFrameImage: public Image
{
public:
	/// constructor
	ExtFrameImage();

	/// virtual destructor
	virtual ~ExtFrameImage() {}

	/// get image type
	virtual string type_str() const;
	
	/**@name attach mandatory objects */
	//@{
		/// attach IMU misalignment
		void attach(IMUMisalignment* p)
		{
			assert(_imu_mis == 0);
			_imu_mis = p;
		}
	//@}
	
	/// calculate exterior orientation
	bool calc_ext_orientation(DPoint3& pcc, DRotMatrix3& N) const;
	
	/// object-to-photo-coords. transformation
	//bool object_to_photo(const DPoint3& obj_c, DPoint3& pho_c) const;
	
	/**@name coordinate transformation */
	//@{
		/**@name object-to-image coords. transformation
		*
		*  @param obj_c (IN) object coordinates
		*  @param img_c (OUT) image coordinates
		*  @param r (IN) row/scanline coordinate (unsed by frame images!)
		*  @return true = no problem, false = error encountered 
		*/
// 		virtual bool object_to_image(const DPoint3& obj_c,
// 			DPoint2& img_c, double r=0) const;

		/// image-to-object coords. transformation (ray-Z-plane intersection)
// 		virtual bool image_to_object(const DPoint2& img_c,
// 			DPoint3& obj_c) const;
	//@}

	/// add constribution to normal equations for all image point observations
	virtual bool normal_eqns(const vector<ObjectPoint*>& obpts,
		Matrix<double>& atpa, Vector<double>& atpl, double& ltpl,
		int& num_obs) const;
	
protected:
	/// IMU misalignment (mandatory)
	IMUMisalignment*	_imu_mis;
};


// non-member functions ...

} // namespace DEP

#endif
