/* $Id: imggeo.h,v 1.12 2007/12/10 06:57:45 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_IMGGEO_INCLUDED
#define DEP_IMGGEO_INCLUDED

#include "station.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{
class ObjectPoint;
class Camera;
class IOPointCon;

/** Transformation parameters class.
 *  Intermediate class only contains the transformation parameters and thier std. devs.
 *  No assumption is made on the imaging camera/sensor
 *  and the mathematical model, e.g. the affine transformation,
 *  the Direct Linear Transformation (DLT) or Polynomial Mapping Function (PMF).
 *
 *  @author Dirk Stallmann
 */
class TransfParams : public Station
{
public:
	/** allocate memory for parameters and std. dev.
	 *  @param sz number of parameters
	 */
	void allocate(size_t sz);

	/// free memory for parameters and std. dev.
	void free();

	/** new size/resize for parameters and std. dev.
	 *  @param sz number of parameters
	 */
	void newsize(size_t sz);

	/** constructor with size
	 *  @param sz number of parameters
	 */
	TransfParams(size_t sz);

	/// virtual destructor
	virtual ~TransfParams();

	/**@name read/write/print transformation parameters */
	//@{
		/// read transformation parameters
		virtual bool read(istream& is);

		/// write transformation parameters
		virtual void write(ostream& os) const;

		/// print transformation parameters
		virtual void print(ostream& os) const;
	//@}

	/**@name overwritten base class AdjObject members */
	//@{
		/// get parameter name
		virtual string uname(size_t i) const;
	
		/// set unknown parameter information
		virtual bool set_upari();
	//@}

	/**@name set and get transformation parameters */
	//@{
		/// get number of transformation parameters
		size_t size() const { return _size; }

		/// set transformation parameters
		void par(int i, double a) { _a[i] = a; }
	
		/// get transformation parameters
		double par(int i) const { return _a[i]; }
		
		/// get transformation parameters
		const double* a() const { return _a; }
	//@}
	
	/// object-to-image coords. transformation
	virtual bool object_to_image(const DPoint3& obj_c, DPoint2& img_c) const = 0;
	
protected:
	/// transformation parameter values
	double*		_a;

	/// std. dev. their of
	double*		_sdev_a;
};


// non-member functions ...

} // namespace DEP

#endif
