/* $Id: poscor.h,v 1.10 2007/07/23 08:51:52 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_POSCOR_INCLUDED
#define DEP_POSCOR_INCLUDED

#include "adjobj.h"

namespace DEP
{
class Image;
class DerivSub;

/** Position correction class.
 *  @author Dirk Stallmann
 */
class PosCorrection : public AdjObject
{
public:
	/// constructor
	PosCorrection();
	
	/**@name read/write/print position correction */
	//@{
		/// read
		bool read(istream& is);

		// write
		//void write(ostream& is, SDevLim& sdev_lim);
	//@}
	
	/**@name overwrite base class members */
	//@{
		/// get adjustment object class name (long description)
		string class_str() const;

		/// get key name
		string key_str() const;

		/// print position correction
		void print(ostream& os) const;

		/// get parameter name
		string uname(size_t i) const;
	
		/// set unknown parameter information (upari)
		bool set_upari();

		// print unknown parameters
		//void print_unknowns(ostream& os) const;

		// print changes of unknowns
		//void print_changes(ostream& os, const Vector<double>& dx) const;
	//@}

	/** set and get reference time
	 */
	//@{
		/// set reference time
		void set_reference_time(double t) { _t0 = t; }

		/// get reference time
		double get_reference_time() const { return _t0; }
	//@}

	/// get position correction
	DPoint3 calc_corr(double t) const;
	
	/// attach to images
	bool attach(vector<Image*>& images);
	
	/** calculate derivatives
	 *
	 */
	void calc_derivs(double t, const DerivSub& deriv_pcc,
		DerivSub& deriv_pc) const;

private:
	/// images IDs that use this set
	vector<string>	_image_ids;

	/// polynomial coeffs.
	vector<DPoint3>	_a;

	/// std. devs. polynom coeffs.
	vector<DPoint3>	_sa;

	/// reference time
	double			_t0;
};

} // namespace DEP

#endif
